#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"

using namespace std;

void
svg_begin(double width, double height) {
    cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    cout << "<svg ";
    cout << "width='" << width << "' ";
    cout << "height='" << height << "' ";
    cout << "viewBox='0 0 " << width << " " << height << "' ";
    cout << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void
svg_end() {
    cout << "</svg>\n";
}

void
svg_text(double left, double baseline, string text) {
    cout << "<text x='" << left
         << "' y='" << baseline
         << "'>" << text << "</text>";
}

void svg_rect(double x, double y, double width, double height,
              string stroke = "red", string fill = "#79553D") {
    cout << "<rect x='" << x
         << "' y='" << y
         << "' width='" << width
         << "' height='" << height
         << "' stroke='" << stroke
         << "' fill='" << fill
         << "' />";
}

void
show_histogram_svg(const vector<size_t>& stolb) {
    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    const auto BLOCK_WIDTH = 10;

    const size_t MAX_ASTERISK = IMAGE_WIDTH - TEXT_WIDTH;

    svg_begin(400, 300);

    size_t maxlen = 0;
    for (size_t len : stolb)
    {
        if (maxlen<len) maxlen = len;
    }

    double top = 0;
    for (size_t bin : stolb) {
        size_t wight = bin;
        if ((maxlen*10) > MAX_ASTERISK){
            if (maxlen != bin) wight = MAX_ASTERISK * (static_cast <double> (bin/maxlen));
            else if (maxlen == bin) wight = MAX_ASTERISK;
        }
        const double bin_width = BLOCK_WIDTH * wight;
        svg_text(TEXT_LEFT, top + TEXT_BASELINE, to_string(bin));
        svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
        top += BIN_HEIGHT;
    }

    svg_end();
}


struct Input {
    vector<double> numbers;
    size_t bucket{};
    size_t number_count{};
};

Input
input_data() {
    Input in;
    cin >> in.number_count;
    cin >> in.bucket;
    in.numbers.resize(in.number_count);
    for (size_t i = 0; i < in.number_count; i++) {
        cin >> in.numbers[i];
    }
    return in;
}

int main()
{
    auto in = input_data();
    auto stolb = make_histogram(in.numbers, in.bucket, in.number_count);
    //show_histogram_text(stolb, in.bucket);
    show_histogram_svg(stolb);

    return 0;
}
